@@ -38,20 +38,25 @@ from .clustering.hac_utils import bic_square_root
 
 from .clustering.cc_iv import ConnectedComponent
 
-from .nnet.wavsets import SeqSet
-from .nnet.seqtoseq import BLSTM
 
 from .model_iv import ModelIV
 
 from .diar import Diar
 from .segmentation import sanity_check
 from .segmentation import bic_linear
-from .segmentation import div_gauss
+from .segmentation import segmentation
 
 from .viterbi import Viterbi
 
 from .scoring import DER
 
+from .nnet import SeqToSeq
+from .nnet import SeqSetSingle
+from .nnet import SeqSetRandomSampler
+from .nnet import SeqSetSampler
+from .nnet import SeqSet
+
+
 
 __author__ = "Sylvain Meignier and Anthony Larcher"
 __copyright__ = "Copyright 2014-2021 Sylvain Meignier and Anthony Larcher"
@@ -60,4 +65,4 @@ __maintainer__ = "Sylvain Meignier"
 __email__ = "sylvain.meignierr@univ-lemans.fr"
 __status__ = "Production"
 __docformat__ = 'reStructuredText'
-__version__ = "0.1.6.3"
+__version__ = "0.2.3"
