@@ -60,13 +60,9 @@ def hac_iv(diar, scores, method="complete", threshold=0.0):
     # print(cluster_list)
     while i < len(link) and link[i, 2] < t:
         # the cluster_list of the 2 clusters
-        logging.debug('c0: {:d} c1: {:d} value: {:.4f}'.format(int(link[i, 0]),
-                                                              int(link[i, 1]),
-                                                              link[i, 2]))
         c0 = cluster_list[int(link[i, 0])]
         c1 = cluster_list[int(link[i, 1])]
-        logging.debug(
-            '\t c0: {} c1: {} value: {:.4f}'.format(c0, c1, link[i, 2]))
+        logging.debug(f"c0: {c0}/{link[i, 0]} c1: {c1}/{link[i, 1]} value: {link[i, 2]:.4f}")
         information(merge, i, c0, c1, link[i, 2])
         if c1 in cluster_dict:
             # c0 is put in c1, and c1 is not empty
@@ -79,7 +75,9 @@ def hac_iv(diar, scores, method="complete", threshold=0.0):
             cluster_dict.pop(c0)
         # add the speaker of the new cluster
         cluster_list.append(c1)
-        ldiar.rename('cluster', [c0], c1)
+        if ldiar.rename('cluster', [c0], c1) == 0:
+            logging.warning(f"cluster: {c0} not in {c1}, no merge done")
+
         i += 1
 
     return ldiar, cluster_dict, merge
