@@ -112,12 +112,12 @@ import logging
 import os
 import re
 import sys
-
+import numpy
 
 from sidekit.sidekit_wrappers import *
 from sidekit.bosaris.idmap import IdMap
 from six import string_types
-from .utils import str2str_normalize
+from .utils import str2str_normalize, path_show_ext
 
 try:
     from sortedcontainers import SortedDict as dict
@@ -133,13 +133,15 @@ class Diar():
     :attr cluster_types: a list object
     :attr segments: a list of Segment object
     """
-    def __init__(self):
+    def __init__(self, format=''):
         self._attributes = AttributeNames()
-        self._attributes.initialize({'show': 0, 'cluster': 1, 'cluster_type': 2,
-                                  'start': 3, 'stop': 4},
-                                    ['empty', 'empty', 'speaker', 0, 0])
+        self._format = format
+        d = {'show': 0, 'cluster': 1, 'cluster_type': 2, 'start': 3, 'stop': 4}
+        l = ['empty', 'empty', 'speaker', 0, 0]
+        self._attributes.initialize(d, l)
         self.cluster_types = ['speaker', 'head']
         self.segments = list()
+        self.add_attributes(format)
 
     def copy_structure(self):
         """
@@ -147,7 +149,7 @@ class Diar():
         and the cluster types. The data is not copy.
         :return: a Diar object
         """
-        tmp_diarization = Diar()
+        tmp_diarization = Diar(self._format)
         tmp_diarization._attributes = copy.deepcopy(self._attributes)
         tmp_diarization.cluster_types = copy.deepcopy(self.cluster_types)
         return tmp_diarization
@@ -237,17 +239,38 @@ class Diar():
                 tmp_diarization.segments.append(copy.deepcopy(seg))
         return tmp_diarization
 
+    def columns(self, attributes):
+        """
+        Rename all values in list old_values into the new value new_value
+        :param attributes: a list of  attributes
+        :return: a list of list
+        """
+        l = list()
+        for segment in self.segments:
+            if isinstance(attributes, list):
+                e = list()
+                for attr in attributes:
+                    e.append(segment[attr])
+                l.append(e)
+            else:
+                l.append(segment[attributes])
+
+        return l
     def rename(self, attribute, old_values, new_value):
         """
         Rename all values in list old_values into the new value new_value
-        :param attribute: speaker of the attribute
-        :param old_values:  list of old values
+        :param attribute: attribute to rename
+        :param old_values:  list of old values to rename
         :param new_value: new value
+        :return number of rename
 
         """
+        nb = 0
         for segment in self.segments:
             if segment[attribute] in old_values or len(old_values) == 0 :
                 segment[attribute] = new_value
+                nb += 1
+        return nb
 
     def _iofi(self, index, attributes, segment):
         """
@@ -333,6 +356,23 @@ class Diar():
         """
         self.segments = list()
 
+    def add_attributes(self, format=''):
+        if format == 'seg':
+            if 'gender' not in self._attributes:
+                self.add_attribut(new_attribut='gender', default='U')
+            if 'env' not in self._attributes:
+                self.add_attribut(new_attribut='env', default='U')
+            if 'channel' not in self._attributes:
+                self.add_attribut(new_attribut='channel', default='U')
+        elif format == 'xseg' or format == 'segx':
+            if 'xvector' not in self._attributes:
+                self.add_attribut(new_attribut='xvector', default='U')
+#            if 'segid' not in self._attributes:
+#                self.add_attribut(new_attribut='segid', default='U')
+        elif format in ['stm', 'mdtm']:
+            if 'gender' not in self._attributes:
+                self.add_attribut(new_attribut='gender', default='U')
+
     def add_attribut(self, new_attribut, default=''):
         """
         Add a attribut
@@ -340,9 +380,15 @@ class Diar():
         :param default: the default value of the attribut
 
         """
-        self._attributes.add(new_attribut, default)
+        if new_attribut not in self._attributes:
+            self._attributes.add(new_attribut, default)
+            #logging.debug("attr diar "+new_attribut)
+            #print(self._attributes.names)
         for seg in self.segments:
-            seg.append(default)
+            if new_attribut not in seg._attributes:
+                seg._attributes.add(new_attribut, default)
+                #logging.debug("attr seg "+new_attribut+' ')
+                seg.append(default)
 
     def del_attribut(self, attribut):
         """
@@ -356,8 +402,31 @@ class Diar():
             i = self._attributes[attribut]
             for seg in self.segments:
                 del seg[i]
+                seg._attributes.delete(attribut)
             self._attributes.delete(attribut)
 
+    def add_duration(self):
+        """
+        Add a "duration" attribute that it the difference between stop - start
+        :return:
+        """
+        self.add_attribut("duration", default=0)
+        for seg in self.segments:
+            seg["duration"] = seg["stop"] - seg["start"]
+
+        return self
+
+    def get_seg_at_time(self, t):
+        """
+
+        :param t: a time in frame number
+        :return: the segment that includes this time stamp
+        """
+        idx = 0
+        while not self.segments[idx]["start"] <= t < self.segments[idx]["stop"]:
+            idx += 1
+        return self.segments[idx]
+
     def _new_row(self, **kwargs):
         """
         Create a new segment initialized with kwargs
@@ -693,15 +762,25 @@ class Diar():
         :param epsilon: the int value to remove
         :return:
         """
-        self.sort(['start'])
-        i = 0
-        if len(self.segments) > 1:
-            self.segments[i]['stop'] = max(self.segments[i]['start'], min(max(self.segments[i + 1]['start'] - (epsilon // 2), 0), self.segments[i]['stop'] + epsilon))
-        i += 1
-        while i < len(self.segments)-1:
-            self.segments[i]['start'] = max(self.segments[i - 1]['stop'], self.segments[i]['start'] - epsilon, 0)
-            self.segments[i]['stop'] = max(self.segments[i]['start'],min(max(self.segments[i + 1]['start'] - (epsilon // 2), 0), self.segments[i]['stop'] + epsilon))
+        index = self.make_index(['show'])
+        for show in index:
+            d = index[show]
+            d.sort(['start'])
+            i = 0
+            if len(d.segments) > 1:
+                d.segments[i]['stop'] = max(d.segments[i]['start'], min(max(d.segments[i + 1]['start'] - (epsilon // 2), 0), d.segments[i]['stop'] + epsilon))
             i += 1
+            while i < len(d.segments)-1:
+                d.segments[i]['start'] = max(d.segments[i - 1]['stop'], d.segments[i]['start'] - epsilon, 0)
+                d.segments[i]['stop'] = max(
+                                            d.segments[i]['start'],
+                                            min(
+                                                max(d.segments[i + 1]['start'] - (epsilon // 2),
+                                                    0),
+                                                d.segments[i]['stop'] + epsilon)
+
+                )
+                i += 1
 
     def collar(self, epsilon=0, warning=False):
         """
@@ -750,7 +829,36 @@ class Diar():
             if segment['start'] < first:
                 first = segment['start']
         return first
-    
+
+    @classmethod
+    def read_segx(cls, filename, normalize_cluster=False, encoding="utf8"):
+        fic = open(filename, 'r', encoding=encoding)
+        diarization = Diar(format='segx')
+        try:
+            for line in fic:
+                line = re.sub('\s+',' ',line)
+                line = line.strip()
+                #logging.debug(line)
+                if line.startswith('#') or line.startswith(';;'):
+                    continue
+                # split line into fields
+                show, start, stop, _, cluster, xvector  = line.split()
+                #                show, start, stop, cluster, segid, xvector  = line.split()
+                #print('###', show, start, stop, cluster, segid, xvector)
+                if normalize_cluster:
+                    cluster = str2str_normalize(cluster)
+                # print(show, tmp, start, length, gender, channel, env, speaker)
+#                diarization.append(show=show, start=int(round(float(start)*100, 0)), stop=int(round(float(stop)*100, 0)),
+#                                   cluster=cluster, xvector=xvector, segid=segid)
+                diarization.append(show=show, start=int(round(float(start) * 100, 0)),
+                                   stop=int(round(float(stop) * 100, 0)),
+                                   cluster=cluster, xvector=xvector)
+        except Exception as e:
+            logging.error(sys.exc_info()[0])
+            #logging.error(line)
+        fic.close()
+        return diarization
+
     @classmethod
     def read_seg(cls, filename, normalize_cluster=False, encoding="utf8"):
         """
@@ -761,13 +869,7 @@ class Diar():
         :return: a diarization object
         """
         fic = open(filename, 'r', encoding=encoding)
-        diarization = Diar()
-        if not diarization._attributes.exist('gender'):
-            diarization.add_attribut(new_attribut='gender', default='U')
-        if not diarization._attributes.exist('env'):
-            diarization.add_attribut(new_attribut='env', default='U')
-        if not diarization._attributes.exist('channel'):
-            diarization.add_attribut(new_attribut='channel', default='U')
+        diarization = Diar(format='seg')
         try:
             for line in fic: 
                 line = re.sub('\s+',' ',line)
@@ -831,9 +933,7 @@ class Diar():
         :return: a diarization object
         """
         fic = open(filename, 'r', encoding=encoding)
-        diarization = Diar()
-        if not diarization._attributes.exist('gender'):
-            diarization.add_attribut(new_attribut='gender', default='U')
+        diarization = Diar(format='stm')
         try:
             for line in fic:
                 line = re.sub('\s+',' ',line)
@@ -882,9 +982,7 @@ class Diar():
         """
 
         fic = open(filename, 'r', encoding=encoding)
-        diarization = Diar()
-        if not diarization._attributes.exist('gender'):
-            diarization.add_attribut(new_attribut='gender', default='U')
+        diarization = Diar(format='mdtm')
         for line in fic:
             line = line.strip()
             line = re.sub('\s+',' ',line)
@@ -912,8 +1010,6 @@ class Diar():
         """
         fic = open(filename, 'r', encoding=encoding)
         diarization = Diar()
-        if not diarization._attributes.exist('gender'):
-            diarization.add_attribut(new_attribut='gender', default='U')
         try:
             name = "uem"
             for line in fic:
@@ -944,16 +1040,21 @@ class Diar():
         """
         fic = open(filename, 'r', encoding=encoding)
         diarization = Diar()
-        if not diarization._attributes.exist('gender'):
-            diarization.add_attribut(new_attribut='gender', default='U')
         try:
             for line in fic:
                 line = re.sub('\s+',' ',line)
                 line = line.strip()
                 if line.startswith('#') or line.startswith(';;'):
                     continue
+                #logging.error("***"+line)
                 # split line into fields
-                spk, show, tmp0, start_str, length, tmp1, tmp2, cluster, tmp3 = line.split()
+                tmp = line.split()
+                spk = tmp[0]
+                show = tmp[1]
+                start_str = tmp[3]
+                length = tmp[4]
+                cluster = tmp[7]
+                
                 if spk == "SPEAKER":
                     start = int(round(float(start_str)*100, 0))
                     stop = start+int(round(float(length)*100, 0))
@@ -1012,7 +1113,7 @@ class Diar():
         return diarization
 
     @classmethod
-    def write_seg(cls, filename, diarization):
+    def write_segx(cls, filename, diarization):
         """
         Write diarization to a segmentation file
         :param filename: the str output filename
@@ -1021,12 +1122,79 @@ class Diar():
         """
         diarization.sort(['show', 'start'])
         fic = open(filename, 'w', encoding="utf8")
+        diarization.add_attributes('segx')
+        for segment in diarization:
+            fic.write(f"{segment['show']} {segment['start']/100.0:.4f} {segment['stop']/100.0:.4f} {segment.duration()/100:.4f} {segment['cluster']} {segment['xvector']} \n")
+        fic.close()
+
+    @classmethod
+    def write_seg(cls, filename, diarization):
+        """
+        Write diarization to a segmentation file
+        :param filename: the str output filename
+        :param diarization: the diarization to write
+
+        """
+        diarization.add_attributes('seg')
+        diarization.sort(['show', 'start'])
+        fic = open(filename, 'w', encoding="utf8")
         for line in Diar.to_string_seg(diarization):
             fic.write(line)
         fic.close()
 
     @classmethod
-    def write_lbl(cls, diarization, label_dir='', label_file_extension='.lbl'):
+    def write_mdtm(cls, filename, diarization):
+        """
+        Write a MDTM file
+        :param filename: the str input filename
+        :param diarization: the diarization to write
+       """
+
+        diarization.add_attributes('mdtm')
+        diarization.sort(['show', 'start'])
+        fic = open(filename, 'w', encoding="utf8")
+        for segment in diarization:
+            if diarization._attributes.exist('gender'):
+                gender = 'adult_male'
+                if segment['gender'] != 'M':
+                    gender = 'adult_female'
+
+            fic.write('{:s} 1 {:.2f} {:.2f} speaker NA {:s} {:s} \n'.format(
+                    segment['show'], segment['start']/100.0, segment.duration()/100.0, gender, segment['cluster']))
+        fic.close()
+
+    @classmethod
+    def write_uem(cls, filename, diarization):
+        """
+        Read a UEM file
+        :param filename: the str input filename
+        :param diarization: the diarization to write
+        """
+        diarization.sort(['show', 'start'])
+        fic = open(filename, 'w', encoding="utf8")
+        for segment in diarization:
+            fic.write('{:s} 1 {:.2f} {:.2f} speaker NA {:s} {:s} \n'.format(
+                segment['show'], segment['start'] / 100.0, segment['stop'] / 100.0))
+        fic.close()
+
+    @classmethod
+    def write_rttm(cls, filename, diarization):
+        """
+        Read rttm file
+        :param filename: str input filename
+        :param diarization: the diarization to write
+        """
+        diarization.add_attributes('mdtm')
+        diarization.sort(['show', 'start'])
+        fic = open(filename, 'w', encoding="utf8")
+        for segment in diarization:
+            fic.write('SPEAKER {:s} 1 {:.2f} {:.2f} <NA> <NA> {:s} <NA> <NA>\n'.format(
+                    segment['show'], segment['start']/100.0, segment.duration()/100.0, segment['cluster']))
+        fic.close()
+
+
+    @classmethod
+    def write_lab(cls, diarization, label_dir='', label_file_extension='.lab'):
         """
         Write diarization to label file
         :param diarization: the diarization to write
@@ -1049,6 +1217,25 @@ class Diar():
                 segment['start'], segment['stop'], segment['cluster']))
         fic.close()
 
+    @classmethod
+    def from_statserver(cls, ss, filtering=None):
+        """
+
+        :param filtering:
+        :param ss:
+        :return:
+        """
+        diarization = Diar()
+
+        # Raw generation of the Diar object
+        for idx, (cluster, show, start, stop) in enumerate(zip(ss.modelset, ss.seget, ss.start, ss.stop)):
+            gender = "u"
+            # We need to convert start and stop in the proper format
+            diarization.append(show=show, cluster=cluster, start=start,
+                               stop=stop, gender=gender)
+
+        return diarization
+
 class Segment(list):
     """
     Class to store the segment informations.
@@ -1065,7 +1252,7 @@ class Segment(list):
 
         """
         list.__init__(self)
-        self._attributes = attributes
+        self._attributes = copy.deepcopy(attributes)
         for item in data:
             self.append(item)
 
@@ -1289,7 +1476,8 @@ class Segment(list):
 
 
 class Index(dict):
-    """Implementation of perl's autovivification feature.
+    """
+    Implementation of perl's auto-vivification feature.
     Thanks to http://stackoverflow.com/questions/651794/whats-the-best-way-to-initialize-a-dict-of-dicts-in-python
     """
 
@@ -1433,4 +1621,22 @@ def rolling_window(a, window):
     return numpy.lib.stride_tricks.as_strided(a, shape=shape, strides=strides)
 
 
-
+def read_diar(fn):
+    dir, name, ext = path_show_ext(fn, shortext=True)
+    if ext.endswith('.segx'):
+        diar = Diar.read_segx(fn)
+    elif ext.endswith('.seg'):
+        diar = Diar.read_seg(fn)
+    elif ext == '.mdtm':
+        diar = Diar.read_mdtm(fn)
+    elif ext == '.rttm':
+        diar = Diar.read_rttm(fn)
+    elif ext == '.uem':
+        diar = Diar.read_uem(fn)
+    else:
+        raise Exception('unknow format: '+ext)
+    #shows = diar.unique('show')
+    #shows.sort()
+    #cluster_list = diar.unique('cluster')
+    #cluster_list.sort()
+    return diar
