@@ -27,9 +27,112 @@ import matplotlib.pyplot as plot
 
 from matplotlib.patches import Rectangle
 from .player import AudioPlayer
-
+import sys
 
 class PlotDiar:
+    """
+    A viewer of diraization
+    """
+    def __init__(self, diar, title=''):
+        """
+
+        :param diar:
+        :param title:
+        :param vgrid:
+        """
+        self.rect_color = (0.0, 0.6, 1.0, 1.0)  # '#0099FF'
+        self.rect_selected_color = (0.75, 0.75, 0, 1.0)  # 'y'
+
+        self.title = title
+        
+        self.height = 5
+        self.diar = diar
+
+        self.map = self.diar.make_index(['show', 'cluster'])
+        #self._draw()
+
+    def draw(self, ax=None):
+        """
+        Draw the diarization
+
+        """
+        minx = sys.maxsize
+        maxx = 0
+        maxy = 0
+
+        if ax is None :
+            ax = plot.gca()
+
+        y = 0
+        labels_pos = []
+        labels = []
+        for show in self.map:
+            for cluster in sorted(self.map[show].keys()):
+                labels.append(cluster)
+                labels_pos.append(y + self.height // 2)
+                for row in self.map[show][cluster]:
+                    x = row['start'] / 100
+                    w = row.duration() / 100
+                    if x < minx:
+                        minx = x
+                    maxx = max(maxx, row['stop'] / 100)
+                    c = self.rect_color
+                    if row._attributes.exist('color'):
+                        c = row['color']
+                        #print("**", c)
+                    rect = Rectangle((x, y), w, self.height, color=c)
+                    ax.add_patch(rect)
+                y += self.height
+            #print([minx, maxx])
+        maxy = y
+        ax.set_yticks(labels_pos) # values
+        ax.set_yticklabels(labels) # labels
+        ax.set_xlim([minx, maxx])
+        ax.set_ylim([0, maxy])
+        
+        for show in self.map:
+            for cluster in self.map[show]:
+                for row in self.map[show][cluster]:
+                    x = row['start'] / 100
+                    ax.plot([x, x], [0, maxy], linestyle=':',
+                                 color='#AAAAAA')
+        
+#        y = maxy
+#        for show in self.map:
+#            for cluster in self.map[show]:
+#                ax.plot([0, maxx], [y, y], linestyle=':',
+#                             color='#AAAAAA')
+#                y -= self.height
+
+        ax.title.set_text(self.title)
+
+        #plot.tight_layout()
+
+
+    @classmethod
+    def _hms(cls, s):
+        """
+        conversion of seconds into hours, minutes and secondes
+        :param s:
+        :return: int, int, float
+        """
+        h = int(s) // 3600
+        s %= 3600
+        m = int(s) // 60
+        s %= 60
+        return '{:d}:{:d}:{:.2f}'.format(h, m, s)
+    
+
+    
+    
+    
+    
+    
+    
+  ##---------------------------  
+    
+    
+class DrawDiar:
     """
     A viewer of segmentation
     """
@@ -167,7 +270,7 @@ class PlotDiar:
                     c = self.rect_color
                     if row._attributes.exist('color'):
                         c = row['color']
-                    rect = plot.Rectangle((x, y), w, self.height,
+                    rect = Rectangle((x, y), w, self.height,
                                           color=c, picker=self.pick)
                     self.ax.add_patch(rect)
                 y += self.height
@@ -344,4 +447,4 @@ class PlotDiar:
         s %= 3600
         m = int(s) // 60
         s %= 60
-        return '{:d}:{:d}:{:.2f}'.format(h, m, s)
+        return '{:d}:{:d}:{:.2f}'.format(h, m, s)
\ No newline at end of file
