@@ -41,8 +41,8 @@ class ModelIV:
         self.tv = None
         self.tv_mean = None
         self.tv_sigma = None
-        self.sn_mean = None
-        self.sn_cov = None
+        self.norm_mean = None
+        self.norm_cov = None
         self.plda_mean = None
         self.plda_f = None
         self.plda_g = None
@@ -81,9 +81,9 @@ class ModelIV:
             print('plda Sigma: ', self.plda_sigma.shape)
             print('plda F: ', self.plda_f.shape)
             print('plda G: ', self.plda_g.shape)
-        if self.sn_mean is not None:
-            print('sn_mean: ', self.sn_mean.shape)
-            print('sn_cov: ', self.sn_cov.shape)
+        if self.norm_mean is not None:
+            print('norm_mean: ', self.norm_mean.shape)
+            print('norm_cov: ', self.norm_cov.shape)
 
     def train(self, feature_server, idmap, normalization=True):
         """
@@ -105,6 +105,36 @@ class ModelIV:
 
         return self.ivectors
 
+    def train_per_speaker(self, show, feature_server, idmap, input_feature_filename=None, normalization=True):
+        """
+
+        :param show:
+        :param feature_server:
+        :param idmap:
+        :param input_feature_filename:
+        :param normalization:
+        :return:
+        """
+        stat = StatServer(idmap, distrib_nb=self.ubm.distrib_nb(), feature_size=self.ubm.dim())
+        stat, _ = stat.sum_stat_per_model() # just used to reduce the size of the statserver
+
+        if input_feature_filename is None:
+            input_feature_filename = idmap.rightids[0]
+
+        stat.accumulate_stat_per_speaker(ubm=self.ubm,
+                                         input_feature_filename=input_feature_filename,
+                                         idmap=idmap,
+                                         feature_server=feature_server)
+
+        fa = FactorAnalyser(mean=self.tv_mean, Sigma=self.tv_sigma, F=self.tv)
+        self.ivectors = fa.extract_ivectors_single(self.ubm, stat)
+
+        if normalization:
+            self.ivectors.spectral_norm_stat1(self.norm_mean[:1], self.norm_cov[:1])
+
+        return self.ivectors
+
+
     def train_per_segment(self, feature_server, idmap, normalization=True):
         """
 
@@ -242,6 +272,7 @@ class ModelIV:
         self.scores = scores
         return scores
 
+
 #def plda_scores_from_diar(model_fn, feature_server, diar, idmap, return_model=False):
 #    model_iv = ModelIV(model_filename=model_fn, feature_server=feature_server, diarization=diar, idmap=idmap)
 
