@@ -25,6 +25,8 @@ Copyright 2014-2021 Sylvain Meignier
 
 import copy
 import numpy as np
+import pandas
+from IPython.display import display, HTML
 
 from scipy.optimize import linear_sum_assignment
 
@@ -178,6 +180,27 @@ class DER:
                     line += f.format(r, h, s)
         return line
 
+    def confusion2pandas(self):
+        """Generate a matrice version of the confusion matrix
+
+        :param all_value: if false, only the confision score upper to 0 is
+        generated
+
+        :return: a str
+        """
+        
+        mat = self.conf/100
+        pd_mat = pandas.DataFrame(data=mat, columns=self.hyp_cluster_list, index=self.ref_cluster_list)
+        return pd_mat
+
+         
+    def confusion2notebook(self):
+        df = self.confusion2pandas()
+        df.loc['Total']= df.sum(numeric_only=True, axis=0)
+        df.loc[:,'Total'] = df.sum(numeric_only=True, axis=1)
+        df.replace(to_replace=0, value='-', inplace=True)
+        display(df)       
+
     def assignment(self):
         """Compute the reference to hypothese association using the hungarian
         algorithm
@@ -190,6 +213,7 @@ class DER:
         name_assigned = dict()
         self.match = 0;
         self.not_assigned = copy.deepcopy(self.hyp_cluster_list)
+        self.not_assigned_ref = copy.deepcopy(self.ref_cluster_list)
 
         for i in range(self.assigned.shape[0]):
             r = self.ref_cluster_list[self.assigned[i, 0]]
@@ -197,14 +221,40 @@ class DER:
             s = self.conf[self.assigned[i, 0], self.assigned[i, 1]]
             if s > 0:
                 self.not_assigned.remove(h)
+                self.not_assigned_ref.remove(r)
                 name_assigned[(r, h)] = s
                 self.match += s
             else:
                 self.assigned[i, 0] = -1
                 self.assigned[i, 1] = -1
 
-        return name_assigned, self.not_assigned
+        return name_assigned, self.not_assigned, self.not_assigned_ref
 
+    def assignment2pandas(self):
+        """Generate a matrice version of the confusion matrix
+
+        :param all_value: if false, only the confision score upper to 0 is
+        generated
+
+        :return: a str
+        """
+        mat = self.conf / 100
+        l = list()
+        name_assigned, not_hyp, not_ref = self.assignment()
+        for key, value in name_assigned.items():
+            l.append((key[0], key[1], value/100))
+        for r in not_ref:
+            idx = self.ref_cluster_list.index(r)
+            l.append((r, 'None', mat[idx,:].sum()))
+        for h in not_hyp:
+            idx = self.hyp_cluster_list.index(h)
+            l.append(('None', h,  mat[:,idx].sum()))        
+        
+        return pandas.DataFrame(data=l, columns=['ref', 'hyp','confusion'])
+        
+    def assignment2notebook(self):
+        display(HTML(self.assignment2pandas().to_html()))
+        
     def set_assignment(self, name_assigned):
         """Set assignment from an external source (from a collection of cluster_list)
 
@@ -256,7 +306,7 @@ class DER:
             line += f.format(r, h, s, -1, -1)
 
         return line
-
+    
     def _correct(self):
         """compute the correct decision
 
@@ -480,7 +530,7 @@ class DER_result:
             if not add_name:
                 lst += self.time(name=name, overlap=overlap)
             else:
-                lst += self.time(name=None)
+                lst += self.time(name=None, overlap=overlap)
 
         #if add_name == False and overlap:
         #    lst += self.overlap(speaker=speaker)
@@ -493,8 +543,8 @@ class DER_result:
         return self.spk_fa+self.spk_miss+self.spk_conf
 
 
-def get_header():
-    return DER_result.header()
+def get_header(speech=True, overlap=False, speaker=True):
+    return DER_result.header(speech=True, overlap=False, speaker=True)
 
 
 def compute_der(hyp, ref, uem=None, collar=0, no_overlap=True):
