@@ -165,8 +165,6 @@ def _split_seg(smooth, segment, min_seg_size, split_size, lst):
     else:
         lst.append(copy.deepcopy(segment))
 
-
-
 def div_gauss(cep, show='empty', win=250, shift=0):
     """
     Segmentation based on gaussian divergence.
@@ -174,9 +172,9 @@ def div_gauss(cep, show='empty', win=250, shift=0):
     The segmentation detects the instantaneous change points corresponding to
     segment boundaries. The proposed algorithm is based on the detection of
     local maxima. It detects the change points through a gaussian divergence
-    (see equation below), computed using Gaussians with diagonal covariance 
-    matrices. The left and right gaussians are estimated over a five-second 
-    window sliding along the whole signal (2.5 seconds for each gaussian, 
+    (see equation below), computed using Gaussians with diagonal covariance
+    matrices. The left and right gaussians are estimated over a five-second
+    window sliding along the whole signal (2.5 seconds for each gaussian,
     given *win* =250 features).
     A change point, i.e. a segment boundary, is present in the middle of the
     window when the gaussian divergence score reaches a local maximum.
@@ -255,6 +253,27 @@ def segmentation(cep, diarization, win_size=250):
 
     return diarization_out
 
+def get_borders(distances, start, stop, show='empty', win=250):
+    """
+
+    :param cep: numpy array of frames
+    :param show: speaker of the show
+    :param win: windows size in number of frames
+    :return: a diarization object (s4d annotation)
+    """
+
+    # find local maximal at + or - win size
+    borders = scipy.signal.argrelmax(distances, order=win)[0].tolist()
+    # append the first and last
+    borders = [0] + borders + [stop-start]
+
+    diarization_out = Diar()
+    spk = 0
+    for i in range(0, len(borders) - 1):
+        diarization_out.append(show=show, start=start+borders[i],
+                               stop=start+borders[i + 1], cluster='S' + str(spk))
+        spk += 1
+    return diarization_out
 
 def bic_linear(cep, diarization, alpha, sr=False):
     """
