@@ -146,7 +146,7 @@ def get_feature_extractor(audio_filename_structure, type_feature_extractor):
                                vad=None,
                                save_param=["energy", "cep", "vad"]
                                )
-    elif type_feature_extractor == 'basic':
+    elif type_feature_extractor == 'basic' or type_feature_extractor == 'sns':
         fe = FeaturesExtractor(audio_filename_structure=audio_filename_structure,
                                feature_filename_structure=None,
                                sampling_frequency=16000,
